package com.tzld.piaoquan.content.understanding.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.TypeReference;
import com.tzld.piaoquan.content.understanding.common.enums.ExceptionEnum;
import com.tzld.piaoquan.content.understanding.common.exception.CommonException;
import com.tzld.piaoquan.content.understanding.dao.mapper.CuPipelineMapper;
import com.tzld.piaoquan.content.understanding.dao.mapper.CuPipelineStepMapper;
import com.tzld.piaoquan.content.understanding.dao.mapper.CuPromptMapper;
import com.tzld.piaoquan.content.understanding.model.dto.PipelineTreeNode;
import com.tzld.piaoquan.content.understanding.model.param.ActionParam;
import com.tzld.piaoquan.content.understanding.model.param.ContentAnalyseParam;
import com.tzld.piaoquan.content.understanding.model.po.CuPipelineStep;
import com.tzld.piaoquan.content.understanding.model.po.CuPipelineStepExample;
import com.tzld.piaoquan.content.understanding.model.po.CuPrompt;
import com.tzld.piaoquan.content.understanding.service.Action;
import com.tzld.piaoquan.content.understanding.service.PipelineService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * @author supeng
 */
@Slf4j
@Service
public class PipelineServiceImpl implements PipelineService {

    @Autowired
    private CuPipelineMapper cuPipelineMapper;

    @Autowired
    private CuPipelineStepMapper cuPipelineStepMapper;

    @Autowired
    private CuPromptMapper cuPromptMapper;

    @Autowired
    private Map<String, Action> actionMap;

    @Autowired
    private MongoTemplate mongoTemplate;

    private static final String COLLECTION_NAME = "content_understanding_info";

    @Override
    public void execute(ContentAnalyseParam param) {
        // 1.获取pipeline配置
        Long pipelineId = param.getPipelineId();
        String content = param.getContent();
        Long videoId = param.getVideoId();
        CuPipelineStepExample example = new CuPipelineStepExample();
        example.createCriteria().andPipelineIdEqualTo(pipelineId);
        List<CuPipelineStep> stepList = cuPipelineStepMapper.selectByExample(example);
        if (Objects.isNull(stepList) || stepList.isEmpty()) {
            throw new CommonException(ExceptionEnum.CONFIG_ERROR, "无配置 pipelineId：" + pipelineId);
        }
        // 2.构建tree结构
        PipelineTreeNode root = buildTree(stepList);
        root.setInput(content);
        // 3.按照步骤执行pipeline每一步动作
        executeTreeNodeBFS(root, videoId);
    }

    public PipelineTreeNode buildTree(List<CuPipelineStep> stepList) {
        Map<Long, PipelineTreeNode> nodeMap = new HashMap<>();
        PipelineTreeNode root = null;

        // 将所有节点放入 Map 中，方便后续查找
        for (CuPipelineStep step : stepList) {
            PipelineTreeNode node = new PipelineTreeNode();
            node.setStep(step);
            nodeMap.put(step.getId(), node);
        }

        // 构建父子关系
        for (CuPipelineStep step : stepList) {
            Long parentId = step.getParentId();
            if (Objects.isNull(parentId) || parentId == 0) {
                root = nodeMap.get(step.getId());
            } else if (nodeMap.containsKey(parentId)) {
                PipelineTreeNode parentNode = nodeMap.get(parentId);
                PipelineTreeNode currentNode = nodeMap.get(step.getId());
                List<PipelineTreeNode> children = parentNode.getChildren();
                if (Objects.isNull(children)) {
                    children = new ArrayList<>();
                    parentNode.setChildren(children);
                }
                children.add(currentNode);
            }
        }
        return root;
    }

    public void executeTreeNodeBFS(PipelineTreeNode root, Long videoId) {
        if (root == null) {
            return;
        }
        Queue<PipelineTreeNode> queue = new LinkedList<>();
        queue.offer(root);

        while (!queue.isEmpty()) {
            PipelineTreeNode currentNode = queue.poll();
            CuPipelineStep step = currentNode.getStep();
            String input = currentNode.getInput() == null ? "" : currentNode.getInput();
            // 执行action
            String result = executeStep(step, input);
            if (Objects.isNull(currentNode.getChildren()) || currentNode.getChildren().isEmpty()) {
                if (Objects.nonNull(result)) {
                    //异常不阻塞其他分支执行
                    try {
                        Map<String, Object> resultMap = JSON.parseObject(result,new TypeReference<Map<String, Object>>() {});
                        resultMap.put("video_id", videoId);
                        mongoTemplate.insert(resultMap, COLLECTION_NAME);
                    } catch (Exception e) {
                        log.error("step execute error step = {}, result = {}", e, JSON.toJSONString(step), result);
                    }
                }
                continue;
            }
            for (PipelineTreeNode child : currentNode.getChildren()) {
                child.setInput(result);
                queue.offer(child);
            }
        }
    }

    /**
     * 执行step 动作
     *
     * @param step
     * @param input
     * @return
     */
    public String executeStep(CuPipelineStep step, String input) {
        CuPrompt cuPrompt = cuPromptMapper.selectByPrimaryKey(step.getPromptId());
        if (Objects.isNull(cuPrompt) || Objects.isNull(cuPrompt.getId())) {
            return null;
        }
        String prompt = String.format(cuPrompt.getPrompt(), input);

        Action action = actionMap.get(step.getAction());
        ActionParam actionParam = new ActionParam();
        actionParam.setInput(prompt);
        actionParam.setPrompt(prompt);
        return action.execute(actionParam);
    }
}
